/*
 *  OFAIServer.java
 *
 *
 *  johann, 25/4/2008
 *
 *  $Id: OFAIServer.java 2820 2001-11-14 17:15:43Z johann $
 */
package at.ofai.gate;

import java.util.*;
import java.net.URL;
import java.io.*;

import gate.*;
import gate.creole.*;
import gate.util.*;

import org.mortbay.jetty.*;
import org.mortbay.jetty.handler.*;
import javax.servlet.http.*;
import javax.servlet.ServletException;

/** 
 * This class is the implementation of the resource OFAIPROCESSDOC.
 */
public class OFAIServer extends AbstractLanguageResource
    implements LanguageResource {

  static final long serialVersionUID = 912203123431412345L;
  protected URL pipelineURL;

  public void setPipelineURL(URL theURL) {
    this.pipelineURL = theURL;
  }

  public URL getPipelineURL() {
    return this.pipelineURL;
  }
  protected Integer port;

  public void setPort(Integer thePort) {
    this.port = thePort;
  }

  public Integer getPort() {
    return this.port;
  }
  public boolean debug = false;
  public boolean debugdocument = false;
  public boolean dorun = false;
  protected OFAIProcessDoc processdoc;
  Server server;

  /** Initialise this resource, and return it. */
  public Resource init() throws ResourceInstantiationException {

    // try to load the processing pipeline from the given file
    try {
      processdoc = new OFAIProcessDoc(getPipelineURL());
    } catch (Exception e) {
      throw new ResourceInstantiationException(e);
    }
    // TODO: create jetty embedded server
    Handler handler = new AbstractHandler() {

      // see http://docs.codehaus.org/display/JETTY/Writing+a+Jetty+Handler
      public synchronized void handle(String target, HttpServletRequest request, HttpServletResponse response, int dispatch)
          throws IOException, ServletException {
        String documentString = request.getParameter("document");
        String commandString = request.getParameter("command");
        String mimetypeString = request.getParameter("mimetype");
        String docnameString = request.getParameter("docname");

        // create an internalized doc from the documentString if we have
        // something in there
        dorun = false;
        if (commandString != null) {
          if (commandString.equalsIgnoreCase("stop")) {
            try {
              if (debug) {
                Err.prln("Debug: shutting down server");
              }
              server.stop();
            } catch (Exception e) {
              Err.prln("Error: during attempt to stop server: " + e);
            }
          } else if (commandString.equalsIgnoreCase("debugon")) {
            Out.prln("Debug: enabled");
            debug = true;
          } else if (commandString.equalsIgnoreCase("debugoff")) {
            Out.prln("Debug: disabled");
            debug = false;
          } else if (commandString.equalsIgnoreCase("run")) {
            dorun = true;
          } else if (commandString.equalsIgnoreCase("debugdocumenton")) {
            debugdocument = true;
            Out.prln("Debug: turning document debugging on");
          } else if (commandString.equalsIgnoreCase("debugdocumentoff")) {
            debugdocument = false;
            Out.prln("Debug: turning docuement debugging off");
          } else {
            Err.prln("Error: unknown command: " + commandString);
          }
        } else {
          dorun = true;
        }
        if (dorun == true) {
          if (documentString != null) {
            FeatureMap parms = gate.Factory.newFeatureMap();
            parms.put("stringContent", documentString);
            if(debugdocument) {
              // create a tmp document from the title and write the string 
              // to it.
              Writer output = new BufferedWriter(new FileWriter(docnameString+"debug.xml"));
              output.write(documentString);
              output.close();
              Out.prln("Debug: written file "+docnameString+".debug.xml");
            }
            Document doc;
            try {
              if (debug) {
                Out.prln("Processing document " + docnameString + " length " + documentString.length());
              }
              doc = (Document) gate.Factory.createResource("gate.corpora.DocumentImpl", parms, null, null);
              doc = processdoc.processDocument(doc);
              //doc = processDocument(doc);
              // serialize the document and send it back!
              
              if (mimetypeString != null) {
                response.setContentType(mimetypeString);
              } else {
                response.setContentType("text/plain");
              }
              response.setStatus(HttpServletResponse.SC_OK);
              documentString = doc.toXml();
              response.getWriter().println(documentString);
              gate.Factory.deleteResource(doc);
              if (debug) {
                Out.prln("Processing OK, result length " + documentString.length());
              }
            } catch (gate.creole.ResourceInstantiationException e) {
              Err.prln("Error: could not create document from the request: " + docnameString);
              doc = null;
              response.setContentType("text/plain");
              response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
              response.getWriter().println("ERROR: could not create document");
            }
          } else {
            Out.prln("Debug: got a request with an empty document or no run command: " + docnameString);
            response.setContentType("text/plain");
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            response.getWriter().println("ERROR: empty");
          }
        }
        ((Request) request).setHandled(true);
      }
    };

    server = new Server(getPort());
    server.setHandler(handler);
    try {
      server.start();
    } catch (Exception e) {
      System.err.println("Exception during server start: " + e);
    }
    return this;
  } // init()


  public Server getServer() {
    return server;
  }

  public void cleanup() {
    try {
      server.stop();
    } catch (Exception e) {
      System.err.println("Exception during server stop: " + e);
    }
  }
  // NOTE: this method exists only to be synchronized 

  public synchronized Document processDocument(Document doc) {
    doc = processdoc.processDocument(doc);
    return doc;
  }
} // class OFAIServer

